import json


def load_flat_json(path):
    """
    Turns json data from:

    {
        "a": 1,
        "b": {
            "c": 2,
            "d": 3
        }
    }

    to:

    {
        "a": 1,
        "b__c": 2,
        "b__d": 3
    }
    """
    with open(path) as fp:
        data = json.load(fp)

    def dive(obj, prefix=()):
        if isinstance(obj, dict):
            for key, value in obj.items():
                yield from dive(value, prefix=[*prefix, key])
        else:
            yield '__'.join(prefix), obj

    return dict(dive(data))



from .envconfig import EnvConfig


__version__ = "1.3.4"

__all__ = ["__version__", "EnvConfig"]


@EnvConfig.parse(bool)
def parse_bool(value: str) -> bool:
    return value.lower() not in ("0", "false", "n", "no", "off")


@EnvConfig.parse(int)
def parse_int(value: str) -> int:
    return int(value)


@EnvConfig.parse(float)
def parse_float(value: str) -> float:
    return float(value)



import os
import sys

from typing import get_type_hints, Type

from .utils import load_flat_json


class EnvVar:
    prefix = ""
    config = {}
    parsers = {}

    def __init__(self, default, *, owner=None, name: str = None):
        self.default = default
        self.__set_name__(owner, name)

    def __set_name__(self, owner, name: str):
        if owner:
            self.type = get_type_hints(owner).get(name, type(self.default))
        self.name = f"__{name}"

    def __get__(self, instance, owner):
        cli_name = self.name.replace("_", "-").lower()
        env_name = f"{self.prefix}{self.name[2:]}"
        cfg_name = self.name[2:].lower()

        try:
            index = sys.argv.index(cli_name)
        except ValueError:
            pass
        else:
            # safely overflow argv with % len - to handle options with no value
            index = (index + 1) % len(sys.argv)
            value = sys.argv[index]
            return self.__parse(value)

        try:
            value = os.environ[env_name]
        except KeyError:
            pass
        else:
            return self.__parse(value)

        try:
            value = self.config[cfg_name]
        except KeyError:
            pass
        else:
            return self.__parse(value)

        return getattr(instance, self.name, self.default)

    def __parse(self, value):
        if self.type not in self.parsers:
            return value

        if isinstance(value, self.type):
            return value

        try:
            return self.parsers[self.type](value)
        except:
            raise AttributeError(
                f'Could not parse "{value}" of type {type(value)} as '
                f"{self.type} using parser {self.parsers[self.type]}"
            )


class EnvConfigMeta(type):
    def __new__(mcs, name, bases, attrs):
        if "_ENV_PREFIX" in attrs:
            EnvVar.prefix = f'{attrs["_ENV_PREFIX"]}_'

        if "_CONFIG_PATH" in attrs and os.path.isfile(attrs["_CONFIG_PATH"]):
            EnvVar.config = load_flat_json(attrs["_CONFIG_PATH"])

        wrapped = {a: EnvVar(v) for a, v in attrs.items() if mcs._should_wrap(a, v)}

        return super().__new__(mcs, name, bases, {**attrs, **wrapped})

    def __setattr__(self, name, value):
        if name == "_ENV_PREFIX":
            EnvVar.prefix = f"{value}_"

        if name == "_CONFIG_PATH" and os.path.isfile(value):
            EnvVar.config = load_flat_json(value)

        elif self._should_wrap(name, value):
            value = EnvVar(value, owner=self, name=name)

        super().__setattr__(name, value)

    @staticmethod
    def _should_wrap(name, value):
        return (
            name.isupper()
            and not name.startswith("_")
            and not isinstance(value, EnvVar)
        )


class EnvConfig(metaclass=EnvConfigMeta):

    def __init__(self):
        # freeze
        self.__dict__.update(
            {k: getattr(self.__class__, k) for k in vars(self.__class__) if k.isupper()}
        )

    @staticmethod
    def parse(type: Type):
        """
        Register a parser for a attribute type.

        Parsers will be used to parse `str` type objects from either
        the commandline arguments or environment variables.

        Args:
            type: the type the decorated function will be responsible
                for parsing a environment variable to.
        """

        def decorator(parser):
            EnvVar.parsers[type] = parser
            return parser

        return decorator



import os
import re
import sys
from distutils.core import setup
from shutil import rmtree

from setuptools import Command

here = os.path.abspath(os.path.dirname(__file__))


with open('sanic_envconfig/__init__.py', 'r') as fd:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
                        fd.read(), re.MULTILINE).group(1)

test_requirements = [
    'pytest',
    'pytest-mock',
    'pytest-cov']


class UploadCommand(Command):
    """Support setup.py upload."""

    description = 'Build and publish the package.'
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(os.path.join(here, 'dist'))
        except OSError:
            pass

        self.status('Building Source and Wheel (universal) distribution…')
        os.system('{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))

        self.status('Uploading the package to PyPi via Twine…')
        os.system('twine upload dist/*')

        self.status('Pushing git tags…')
        os.system('git tag v{0}'.format(version))
        os.system('git push --tags')

        sys.exit()


setup(
    name='sanic_envconfig',
    version=version,
    packages=['sanic_envconfig'],
    url='https://github.com/jamesstidard/sanic-envconfig',
    license='MIT',
    author='James Stidard',
    author_email='jamesstidard@gmail.com',
    description='Pull environment and commandline variables into your sanic config class.',
    keywords='sanic config environment variables extension',
    platforms=['any'],
    tests_require=test_requirements,
    extras_require={'test': test_requirements},
    classifiers=(
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
    ),
    cmdclass={
        'upload': UploadCommand,
    },
)
